@@ -3,11 +3,11 @@
 # pylint: disable=line-too-long
 # pylint: disable=unused-import
 
-import json
-import time
-from dataclasses import dataclass, field
-from typing import List
-from typing import Iterable, Union
+# import json
+# import time
+from dataclasses import dataclass
+# from typing import List
+# from typing import Iterable, Union
 
 
 # import colemen_utils as c
@@ -20,10 +20,15 @@ from typing import Iterable, Union
 # import apricity.objects.Log as log
 # from apricity.susurrus.request import Request
 # from settings.types import _Union
+from colemen_config import _db_column_type,_db_column_sql_data_type,_db_column_validation_data_type,_db_column_form_data_type,_db_mysql_database_type,_db_table_type,_db_relationship_type
+
+
 import colemen_utilities.type_utils as _type
 import colemen_utilities.dict_utils as _obj
 import colemen_utilities.database_utils.MySQL.Column.column_utils as _u
-from colemen_config import _db_column_type,_db_column_sql_data_type,_db_column_validation_data_type,_db_column_form_data_type,_db_mysql_database_type,_db_table_type
+import colemen_utilities.database_utils.MySQL.Relationship.Relationship as _rel
+import colemen_utilities.console_utils as _con
+_log = _con.log
 
 
 
@@ -35,9 +40,12 @@ class Column:
     data:_db_column_sql_data_type = None
     validation:_db_column_validation_data_type = None
     form:_db_column_form_data_type = None
+    _relationship:_db_relationship_type = None
+    '''The relationship instance associated to this column, if one exists.'''
 
     def __init__(self,database:_db_mysql_database_type,table:_db_table_type,data=None) -> None:
         self.database = database
+        self._dbm = self.database._dbm
         self.table = table
         # self.schema_name = db.name
         self._column_name:str = None
@@ -47,19 +55,23 @@ class Column:
         self.form:_db_column_form_data_type = self.form_data(self)
         self.validation:_db_column_validation_data_type = self.validation_data(self)
 
-
-        if data is not None:
+        # @Mstep [IF] if a data dicitonary was provided
+        if isinstance(data,(dict)):
             sql_data = data
+            # @Mstep [IF] if sql_data is in the dict, the data is from a cache file.
             if 'sql_data' in data:
-                # print(f"sql_data found in column dictionary")
+                # @Mstep [] set sql_data to the dicitonary's value
                 sql_data = data['sql_data']
+            # @Mstep [] have the data instance populate from the sql_data
             self.data.populate_from_meta(sql_data)
+            
             if 'form_data' in data:
-                # print(f"form_data found in column dictionary")
                 self.form.populate_from_dict(data['form_data'])
             if 'validation' in data:
                 self.validation.populate_from_dict(data['validation'])
 
+        # @Mstep [] register this column with the database.
+        self.database.register(self)
 
 
 
@@ -83,6 +95,8 @@ class Column:
             "validation":self.validation.summary(),
             "sql_data":self.data.summary(),
         }
+        if self.relationship is not None:
+            value["relationship"] = self.relationship.summary
         return value
 
 
@@ -148,7 +162,32 @@ class Column:
         }
         return value
 
+    @property
+    def relationship(self)->_db_relationship_type:
+        '''
+            Get this Column's relationship instance, if it has one.
 
+            `default`:None
+
+            Meta
+            ----------
+            `@author`: Colemen Atwood
+            `@created`: 12-16-2022 11:05:43
+            `@memberOf`: Column
+            `@property`: relationship
+        '''
+        value = self._relationship
+        if value is None:
+            if self.data.is_foreign_key is True:
+                _log("Column.relationship --- Relationship Found","magenta")
+                value = _rel.Relationship(
+                    self.database,
+                    self.table,
+                    self,
+                    self.data.foreign_key_data
+                )
+                self._relationship = value
+        return value
 
     @dataclass
     class sql_data:
@@ -175,6 +214,7 @@ class Column:
         generation_expression:str = None
         hash_id_prefix:str = None
         is_fulltext_indexed:bool = False
+        '''True if this column has a fulltext index.'''
         is_timestamp:bool = False
         '''True if this column's name is "timestamp", meaning its used for tracking when the row was created.'''
         is_modified_timestamp:bool = False
@@ -212,6 +252,7 @@ class Column:
 
         _is_hash_id:bool = False
 
+
         def __init__(self,column:_db_column_type):
             self._column:_db_column_type = column
 
@@ -270,6 +311,14 @@ class Column:
                             self.foreign_table_name = self.constraint['REFERENCED_TABLE_NAME']
                             self.foreign_column_name = self.constraint['REFERENCED_COLUMN_NAME']
                             self.foreign_key_constraint_name = self.constraint['CONSTRAINT_NAME']
+                            if self._column.table.get_relationship(self.foreign_key_constraint_name) is None:
+                                self._column._relationship = _rel.Relationship(
+                                    self._column.database,
+                                    name=self.constraint['CONSTRAINT_NAME'],
+                                    foreign_table_schema = self.constraint['REFERENCED_TABLE_SCHEMA'],
+                                    foreign_table_name = self.constraint['REFERENCED_TABLE_NAME'],
+                                    foreign_column_name = self.constraint['REFERENCED_COLUMN_NAME'],
+                                )
                     # self.is_foreign_key = True
 
 
@@ -310,6 +359,12 @@ class Column:
                 if hasattr(self._column.validation,k):
                     setattr(self._column.validation,k,v)
 
+            if isinstance(self._column.table.raw_fulltext_indexes,(list)):
+                for ft in self._column.table.raw_fulltext_indexes:
+                    if ft['column_name'] == self.column_name:
+                        self.is_fulltext_indexed = True
+            
+            
         @property
         def is_nullable(self)->bool:
             '''
@@ -440,6 +495,41 @@ class Column:
             '''
             self._is_foreign_key = value
 
+        @property
+        def foreign_key_data(self):
+            '''
+                Get this Column's foreign_key_data
+                If this column is a foreign key, this will return a dictionary of the foreign key data:
+                
+                - schema_name
+                - table_name
+                - column_name
+                - foreign_table_schema
+                - foreign_table_name
+                - foreign_column_name
+                - foreign_key_constraint_name
+
+                `default`:None
+
+
+                Meta
+                ----------
+                `@author`: Colemen Atwood
+                `@created`: 12-16-2022 11:08:08
+                `@memberOf`: Column
+                `@property`: foreign_key_data
+            '''
+            value = {
+                "schema_name":self._column.database.name,
+                "table_name":self._column.table.table_name,
+                "column_name":self.column_name,
+                "foreign_table_schema":self.foreign_table_schema,
+                "foreign_table_name":self.foreign_table_name,
+                "foreign_column_name":self.foreign_column_name,
+                "foreign_key_constraint_name":self.foreign_key_constraint_name,                
+            }
+            return value
+
         @property
         def default(self):
             '''
@@ -720,8 +810,9 @@ class Column:
                     value['check_with'] = is_email.__name__
 
             if self._data.is_hash_id:
-                max_nonce = self._data.character_maximum_length - len(self._data.hash_id_prefix)
-                value['regex'] = f'{self._data.hash_id_prefix}_[a-zA-Z0-9]{{1,{max_nonce}}}'
+                if isinstance(self._data.hash_id_prefix,(str)):
+                    max_nonce = self._data.character_maximum_length - len(self._data.hash_id_prefix)
+                    value['regex'] = f'{self._data.hash_id_prefix}_[a-zA-Z0-9]{{1,{max_nonce}}}'
 
 
 
