@@ -27,7 +27,7 @@ from collections.abc import MutableMapping
 
 def set_defaults(default_vals, obj,**kwargs):
     '''
-        Sets default values on the dict provided, if they do not already exist or 
+        Sets default values on the dict provided, if they do not already exist or
         if the value is None.
 
         ----------
@@ -57,7 +57,7 @@ def set_defaults(default_vals, obj,**kwargs):
         `method_name`: set_defaults
     '''
     replace_null = get_kwarg(['replace null'],False,(bool),**kwargs)
-    
+
     for k, v in default_vals.items():
         if replace_null:
             if k in obj:
@@ -584,7 +584,7 @@ def reorder_keys(data:dict,**kwargs)->dict:
         If a `keys` list/dict is provided it will sort the `data` keys to match it.
         Any keys are exit in `data` but NOT in `keys` will be appended to the end
         in alphabetical order.
-        
+
         If `alpha` is True, it will sort the dictionary alphabetically.
 
         ----------
@@ -715,6 +715,12 @@ def flatten(d, parent_key='', sep='.'):
         `data` {dict}
             The dictionary to flatten.
 
+        [`parent_key`=""] {str}
+            A prefix to apply to all keys
+
+        [`sep`="."] {str}
+            The separator used to indicate nesting
+
         Return {dict}
         ----------------------
         The flattened dictionary.
@@ -760,7 +766,7 @@ def has_method(class_instance,method_name):
         -------------------------
         `class_instance` {class}
             The class to test.
-            
+
         `method_name` {str}
             The name of the method to search for
 
@@ -789,7 +795,7 @@ def has_method(class_instance,method_name):
 def has_attr(class_instance,keys):
     '''
         Check if the class provided has an attribute matching a key.
-        
+
         This is identical to calling "hasattr" except it accepts a list of keys and returns the matching one.
 
         ----------
@@ -798,7 +804,7 @@ def has_attr(class_instance,keys):
         -------------------------
         `class_instance` {any}
             The class to search within.
-            
+
         `keys` {list,str}
             The key or list of keys to search for.
 
@@ -819,10 +825,10 @@ def has_attr(class_instance,keys):
 
 
     keys = _lu.force_list(keys)
-    
-    
+
+
     for key in keys:
         if hasattr(class_instance,key):
             return key
-        
+
     return False
